/*
 * Generate a P3DC model file of a cone
 */
#include <stdio.h>
#include <math.h>

/* The magic number in graphics ... */
#define PI			(3.14159265358979323846)
#define TWOPI		(2*PI)

#define EQZ(a)	(fabs(a) < .000001)

int sides = 32;

double
fx(int a) {
	double r;
	r = 0.5 * sin ((double)(a)/(double)(sides) * TWOPI);
	if (EQZ(r)) r = 0;
	return r;
}

double
fy(int a) {
	double r;
	r = 0.5 * cos ((double)(a)/(double)(sides) * TWOPI);
	if (EQZ(r)) r = 0;
	return r;
}

/* Take a number between 1 and 32 and return a number between 0 and 1 */
double
tx(int a) {
	double r;
	r = (double) a / (double)(sides);
	if (EQZ(r)) return 0.5;
	return (r * 255.5);
}

void
main(int argc, char *argv[]) {
	double x, y;
	int i;
	int vtx_num = 0;


	if (argc != 1) {
		sides = atoi(argv[1]);
		if (sides == 0)
			exit(1);
	}
	
	printf("#\n# Cone Model - autogenerated w/%d segments\n", sides);
	printf("#\n# This model was generated automatically by a program\n");
	printf("# cyl.c. This program creates a 32 segment cylinder\n");
	printf("# that is centered on the origin.\n#\n");
	printf("# base_texture -- wraps around the circumference.\n");
	printf("# bottom_texture -- covers the -Z end of the cylinder.\n");
	printf("#\n");
	printf("model cone%d {\n", sides);
	printf("	origin 0, 0, 0;\n");
	printf("	color base = 255, 0, 0; \n");
	printf("	texture base_texture = DECAL ; \n");
	printf("	texture bottom_texture = DECAL ;\n");
	for (i = 0; i < sides; i++) {
		x = fx(i);
		y = fy(i);
		++vtx_num;
		printf("	Vertex VB%d = %g, %g, %g ;\n", vtx_num, x, y, -0.5);
	}
	printf("	Vertex VT = 0.0, 0.0, 0.5;\n");

	/* 
	 * Generate the bottom
	 */
	printf("	Face <bottom_texture> (base) ");
	for (i = sides; i > 1; i--) {
		printf("VB%d <%g, %g>, ", i, 
						((fx(i)+0.5) * 255.5), ((fy(i)+0.5) * 255.5));
	}
	printf("VB1 <%g, %g>;\n ", 
						((fx(1)+0.5) * 255.5), ((fy(1)+0.5) * 255.5));

	for (i = 1; i < sides; i++) {
		printf("	Face <base_texture> (base) ");
		printf("VB%d [%g, %g, 0] <%g, 0.5>,  ",
					i+1, fx(i), fy(i), tx(i));
		printf("VT [%g, %g, 0.25] <%g, 255.5>, ",
					fx(i-1), fy(i-1), tx(i-1));
		printf("VB%d [%g, %g, 0] <%g, 0.5>;\n ",
					i, fx(i-1), fy(i-1), tx(i-1));
	}
	x = fx(sides-1);
	y = fy(sides-1);
	printf("	Face <base_texture> (base) ");
	printf("VB1 [%g, %g, 0] <255.5, 0.5>, ", fx(0), fy(0));
	printf("VT [%g, %g, 0] <%g, 255.5>, VB%d [%g, %g, 0] <%g, 0.5>;\n", 
				x, y, tx(sides-1), sides, x, y, tx(sides-1));
	printf("}\n");
}

